@@ -26,6 +26,7 @@
 #include <QtGui/QBrush>
 #include <QtCore/QFile>
 #include <QtCore/QFileInfo>
+#include <QtGui/QPainter>
 
 // KDE
 #include <KColorScheme>
@@ -90,26 +91,26 @@ const char* const ColorScheme::colorNames[TABLE_COLORS] =
 };
 const char* const ColorScheme::translatedColorNames[TABLE_COLORS] =
 {
-    I18N_NOOP("Foreground"),
-    I18N_NOOP("Background"),
-    I18N_NOOP("Color 1"),
-    I18N_NOOP("Color 2"),
-    I18N_NOOP("Color 3"),
-    I18N_NOOP("Color 4"),
-    I18N_NOOP("Color 5"),
-    I18N_NOOP("Color 6"),
-    I18N_NOOP("Color 7"),
-    I18N_NOOP("Color 8"),
-    I18N_NOOP("Foreground (Intense)"),
-    I18N_NOOP("Background (Intense)"),
-    I18N_NOOP("Color 1 (Intense)"),
-    I18N_NOOP("Color 2 (Intense)"),
-    I18N_NOOP("Color 3 (Intense)"),
-    I18N_NOOP("Color 4 (Intense)"),
-    I18N_NOOP("Color 5 (Intense)"),
-    I18N_NOOP("Color 6 (Intense)"),
-    I18N_NOOP("Color 7 (Intense)"),
-    I18N_NOOP("Color 8 (Intense)")
+    I18N_NOOP2("@item:intable palette", "Foreground"),
+    I18N_NOOP2("@item:intable palette", "Background"),
+    I18N_NOOP2("@item:intable palette", "Color 1"),
+    I18N_NOOP2("@item:intable palette", "Color 2"),
+    I18N_NOOP2("@item:intable palette", "Color 3"),
+    I18N_NOOP2("@item:intable palette", "Color 4"),
+    I18N_NOOP2("@item:intable palette", "Color 5"),
+    I18N_NOOP2("@item:intable palette", "Color 6"),
+    I18N_NOOP2("@item:intable palette", "Color 7"),
+    I18N_NOOP2("@item:intable palette", "Color 8"),
+    I18N_NOOP2("@item:intable palette", "Foreground (Intense)"),
+    I18N_NOOP2("@item:intable palette", "Background (Intense)"),
+    I18N_NOOP2("@item:intable palette", "Color 1 (Intense)"),
+    I18N_NOOP2("@item:intable palette", "Color 2 (Intense)"),
+    I18N_NOOP2("@item:intable palette", "Color 3 (Intense)"),
+    I18N_NOOP2("@item:intable palette", "Color 4 (Intense)"),
+    I18N_NOOP2("@item:intable palette", "Color 5 (Intense)"),
+    I18N_NOOP2("@item:intable palette", "Color 6 (Intense)"),
+    I18N_NOOP2("@item:intable palette", "Color 7 (Intense)"),
+    I18N_NOOP2("@item:intable palette", "Color 8 (Intense)")
 };
 
 ColorScheme::ColorScheme()
@@ -117,11 +118,14 @@ ColorScheme::ColorScheme()
     _table = 0;
     _randomTable = 0;
     _opacity = 1.0;
+    setWallpaper(QString());
 }
+
 ColorScheme::ColorScheme(const ColorScheme& other)
       : _opacity(other._opacity)
        ,_table(0)
        ,_randomTable(0)
+       ,_wallpaper(other._wallpaper)
 {
     setName(other.name());
     setDescription(other.description());
@@ -164,7 +168,7 @@ void ColorScheme::setColorTableEntry(int index , const ColorEntry& entry)
         for (int i=0;i<TABLE_COLORS;i++)
             _table[i] = defaultTable[i];
     }
-    
+
     _table[index] = entry; 
 }
 ColorEntry ColorScheme::colorEntry(int index , uint randomSeed) const
@@ -181,14 +185,14 @@ ColorEntry ColorScheme::colorEntry(int index , uint randomSeed) const
         !_randomTable[index].isNull() )
     {
         const RandomizationRange& range = _randomTable[index];
-      
+
 
         int hueDifference = range.hue ? (qrand() % range.hue) - range.hue/2 : 0;
         int saturationDifference = range.saturation ? (qrand() % range.saturation) - range.saturation/2 : 0;
         int  valueDifference = range.value ? (qrand() % range.value) - range.value/2 : 0;
 
         QColor& color = entry.color;
-      
+
         int newHue = qAbs( (color.hue() + hueDifference) % MAX_HUE );
         int newValue = qMin( qAbs(color.value() + valueDifference) , 255 );
         int newSaturation = qMin( qAbs(color.saturation() + saturationDifference) , 255 );
@@ -209,7 +213,7 @@ bool ColorScheme::randomizedBackgroundColor() const
 }
 void ColorScheme::setRandomizedBackgroundColor(bool randomize)
 {
-    // the hue of the background colour is allowed to be randomly 
+    // the hue of the background color is allowed to be randomly
     // adjusted as much as possible.
     //
     // the value and saturation are left alone to maintain read-ability
@@ -270,6 +274,7 @@ void ColorScheme::read(KConfig& config)
 
     _description = i18n(description.toUtf8());
     _opacity = configGroup.readEntry("Opacity",qreal(1.0));
+    setWallpaper(configGroup.readEntry("Wallpaper", QString()));
 
     for (int i=0 ; i < TABLE_COLORS ; i++)
     {
@@ -282,7 +287,8 @@ void ColorScheme::write(KConfig& config) const
 
     configGroup.writeEntry("Description",_description);
     configGroup.writeEntry("Opacity",_opacity);
-    
+    configGroup.writeEntry("Wallpaper", _wallpaper->path());
+
     for (int i=0 ; i < TABLE_COLORS ; i++)
     {
         RandomizationRange random = _randomTable != 0 ? _randomTable[i] : RandomizationRange();
@@ -300,12 +306,12 @@ QString ColorScheme::translatedColorNameForIndex(int index)
 {
     Q_ASSERT( index >= 0 && index < TABLE_COLORS );
 
-    return i18n(translatedColorNames[index]);
+    return i18nc("@item:intable palette", translatedColorNames[index]);
 }
 void ColorScheme::readColorEntry(KConfig& config , int index)
 {
     KConfigGroup configGroup(&config,colorNameForIndex(index));
-    
+
     ColorEntry entry;
 
     entry.color = configGroup.readEntry("Color",QColor());
@@ -350,6 +356,58 @@ void ColorScheme::writeColorEntry(KConfig& config , const QString& colorName, co
     }
 }
 
+void ColorScheme::setWallpaper(const QString& path)
+{
+    _wallpaper.attach(new ColorSchemeWallpaper(path));
+}
+
+ColorSchemeWallpaper::Ptr ColorScheme::wallpaper() const
+{
+    return _wallpaper;
+}
+
+ColorSchemeWallpaper::ColorSchemeWallpaper(const QString& path)
+    : _path(path),
+      _picture(0)
+{
+}
+
+ColorSchemeWallpaper::~ColorSchemeWallpaper()
+{
+    delete _picture;
+}
+
+void ColorSchemeWallpaper::load()
+{
+    if (_path.isEmpty())
+        return;
+
+    // Create and load original pixmap
+    if (!_picture)
+        _picture = new QPixmap();
+
+    if (_picture->isNull())
+        _picture->load(_path);
+}
+
+bool ColorSchemeWallpaper::isNull() const
+{
+    return _path.isEmpty();
+}
+
+bool ColorSchemeWallpaper::draw(QPainter& painter, const QRect& rect)
+{
+    if (!_picture || _picture->isNull())
+        return false;
+
+    painter.drawTiledPixmap(rect, *_picture, rect.topLeft());
+    return true;
+}
+
+QString ColorSchemeWallpaper::path() const
+{
+    return _path;
+}
 
 // 
 // Work In Progress - A color scheme for use on KDE setups for users
@@ -460,7 +518,7 @@ bool KDE3ColorSchemeReader::readColorLine(const QString& line,ColorScheme* schem
         return false;
     if (list.first() != "color")
         return false;
-    
+
     int index = list[1].toInt();
     int red = list[2].toInt();
     int green = list[3].toInt();
@@ -519,7 +577,7 @@ void ColorSchemeManager::loadAllColorSchemes()
     int failed = 0;
 
     QList<QString> nativeColorSchemes = listColorSchemes();
-    
+
     QListIterator<QString> nativeIter(nativeColorSchemes);
     while ( nativeIter.hasNext() )
     {
@@ -570,7 +628,7 @@ bool ColorSchemeManager::loadKDE3ColorScheme(const QString& filePath)
         delete scheme;
         return false;
     }
-    
+
     QFileInfo info(filePath);
 
     if ( !_colorSchemes.contains(info.baseName()) )
@@ -600,7 +658,7 @@ bool ColorSchemeManager::loadColorScheme(const QString& filePath)
         return false;
 
     QFileInfo info(filePath);
-    
+
     KConfig config(filePath , KConfig::NoGlobals);
     ColorScheme* scheme = new ColorScheme();
     scheme->setName(info.baseName());
@@ -621,7 +679,7 @@ bool ColorSchemeManager::loadColorScheme(const QString& filePath)
     {
         kDebug() << "color scheme with name" << scheme->name() << "has already been" <<
             "found, ignoring.";
-        
+
         delete scheme;
     }
 
@@ -632,7 +690,7 @@ QList<QString> ColorSchemeManager::listKDE3ColorSchemes()
     return KGlobal::dirs()->findAllResources("data",
                                              "konsole/*.schema",
                                               KStandardDirs::NoDuplicates);
-    
+
 }
 QList<QString> ColorSchemeManager::listColorSchemes()
 {
