@@ -19,7 +19,6 @@
 
 // Own
 #include "MainWindow.h"
-#include "SessionManager.h"
 
 // Qt
 #include <QtGui/QBoxLayout>
@@ -55,6 +54,7 @@
 #include "Session.h"
 #include "ViewManager.h"
 #include "ViewSplitter.h"
+#include "SessionManager.h"
 
 using namespace Konsole;
 
@@ -70,9 +70,12 @@ MainWindow::MainWindow()
  : KXmlGuiWindow() ,
    _bookmarkHandler(0),
    _pluggedController(0),
-   _menuBarVisibilitySet(false)
+   _menuBarInitialVisibility(true),
+   _menuBarInitialVisibilitySet(false),
+   _menuBarInitialVisibilityApplied(false)
 {
-    if (useTransparency()) {
+    if ( useTransparency() )
+    {
         setAttribute(Qt::WA_TranslucentBackground);
         setAttribute(Qt::WA_NoSystemBackground, false);
     }
@@ -81,17 +84,17 @@ MainWindow::MainWindow()
     setupActions();
 
     // create view manager
-        _viewManager = new ViewManager(this,actionCollection());
+    _viewManager = new ViewManager(this,actionCollection());
     connect( _viewManager , SIGNAL(empty()) , this , SLOT(close()) );
     connect( _viewManager , SIGNAL(activeViewChanged(SessionController*)) , this ,
             SLOT(activeViewChanged(SessionController*)) );
     connect( _viewManager , SIGNAL(unplugController(SessionController*)) , this ,
             SLOT(disconnectController(SessionController*)) );
-    connect( _viewManager , SIGNAL(viewPropertiesChanged(const QList<ViewProperties*>&)) ,
-           bookmarkHandler() , SLOT(setViews(const QList<ViewProperties*>&)) );
+    connect( _viewManager , SIGNAL(viewPropertiesChanged(QList<ViewProperties*>)) ,
+           bookmarkHandler() , SLOT(setViews(QList<ViewProperties*>)) );
 
     connect( _viewManager , SIGNAL(setMenuBarVisibleRequest(bool)) , this ,
-            SLOT(setMenuBarVisibleOnce(bool)) );
+            SLOT(setMenuBarInitialVisibility(bool)) );
     connect( _viewManager , SIGNAL(setSaveGeometryOnExitRequest(bool)) , this ,
 	    SLOT(setSaveGeometryOnExit(bool)) );
     connect( _viewManager , SIGNAL(updateWindowIcon()) , this ,
@@ -108,6 +111,7 @@ MainWindow::MainWindow()
     // menu items - to avoid conflicting with Alt+[Letter] shortcuts
     // in terminal applications
     KAcceleratorManager::setNoAccel(menuBar());
+
     // create menus
     createGUI();
     // remove accelerators for standard menu items (eg. &File, &View, &Edit)
@@ -135,15 +139,15 @@ void MainWindow::removeMenuAccelerators()
         menuItem->setText(itemText);
     }
 }
-void MainWindow::setMenuBarVisibleOnce(bool visible)
+void MainWindow::setMenuBarInitialVisibility(bool visible)
 {
-    if (_menuBarVisibilitySet || menuBar()->isTopLevelMenu() )
+    // Make sure the 'initial' visibility is set only once.
+    if ( _menuBarInitialVisibilitySet )
         return;
 
-    menuBar()->setVisible(visible);
-    _toggleMenuBarAction->setChecked(visible);
-
-    _menuBarVisibilitySet = true;
+    // The initial visibility of menubar will not be applied until showEvent(),
+    _menuBarInitialVisibility    = visible ;
+    _menuBarInitialVisibilitySet = true;
 }
 
 void MainWindow::setSaveGeometryOnExit(bool save)
@@ -155,13 +159,10 @@ void MainWindow::correctShortcuts()
 {
     // replace F1 shortcut for help contents
     QAction* helpAction = actionCollection()->action("help_contents");
-
     Q_ASSERT( helpAction );
-
     helpAction->setShortcut(QKeySequence());
-   
+
     // replace Ctrl+B shortcut for bookmarks
-    // TODO - Make this configurable
     QAction* bookmarkAction = actionCollection()->action("add_bookmark");
     Q_ASSERT(bookmarkAction);
     bookmarkAction->setShortcut(QKeySequence(Qt::CTRL+Qt::SHIFT+Qt::Key_B));
@@ -200,9 +201,9 @@ void MainWindow::activeViewChanged(SessionController* controller)
 {
     // associate bookmark menu with current session
     bookmarkHandler()->setActiveView(controller);
-    disconnect( bookmarkHandler() , SIGNAL(openUrl(const KUrl&)) , 0 , 0 );
-    connect( bookmarkHandler() , SIGNAL(openUrl(const KUrl&)) , controller ,
-             SLOT(openUrl(const KUrl&)) );
+    disconnect( bookmarkHandler() , SIGNAL(openUrl(KUrl)) , 0 , 0 );
+    connect( bookmarkHandler() , SIGNAL(openUrl(KUrl)) , controller ,
+             SLOT(openUrl(KUrl)) );
 
     if ( _pluggedController )
         disconnectController(_pluggedController);
@@ -247,6 +248,7 @@ IncrementalSearchBar* MainWindow::searchBar() const
 void MainWindow::setupActions()
 {
     KActionCollection* collection = actionCollection();
+    KAction* action = 0;
 
     // File Menu
     _newTabMenuAction = new KActionMenu(KIcon("tab-new"), i18n("&New Tab"), collection);
@@ -256,7 +258,7 @@ void MainWindow::setupActions()
     connect(_newTabMenuAction, SIGNAL(triggered()), this, SLOT(newTab()));
     collection->addAction("new-tab", _newTabMenuAction);
 
-    KAction* action = collection->addAction("new-window");
+    action = collection->addAction("new-window");
     action->setIcon( KIcon("window-new") );
     action->setText( i18n("New &Window") );
     action->setShortcut( QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_N) );
@@ -264,29 +266,20 @@ void MainWindow::setupActions()
     connect( action , SIGNAL(triggered()) , this , SLOT(newWindow()) );
 
     action = KStandardAction::quit( this , SLOT(close()) , collection );
-    // the default shortcut for quit is typically Ctrl+[Some Letter, usually Q] but that is reserved for
-    // use by terminal applications
+    // the default shortcut for quit is typically Ctrl+[Some Letter, usually Q]
+    // but that is reserved for use by terminal applications
     action->setShortcut( QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_Q) );
 
     // Bookmark Menu
     KActionMenu* bookmarkMenu = new KActionMenu(i18n("&Bookmarks") , collection );
     _bookmarkHandler = new BookmarkHandler( collection , bookmarkMenu->menu() , true , this );
     collection->addAction("bookmark" , bookmarkMenu);
-
     connect( _bookmarkHandler , SIGNAL(openUrls(QList<KUrl>)) , this , SLOT(openUrls(QList<KUrl>)) );
 
-    //TODO: The 'Add Bookmark' menu action currently has a Ctrl+B shortcut by
-    //      default which cannot be overridden
-    //NOTE: This is currently handled by correctShortcuts()
-
     // View Menu
     _toggleMenuBarAction = KStandardAction::showMenubar(menuBar(), SLOT(setVisible(bool)), collection);
     _toggleMenuBarAction->setShortcut( QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_M) );
 
-    // Hide the Show/Hide menubar item if the menu bar is a MacOS-style menu bar
-    if ( menuBar()->isTopLevelMenu() )
-        _toggleMenuBarAction->setVisible(false);
-
     // Full Screen
     action = KStandardAction::fullScreen(this, SLOT(viewFullScreen(bool)), this, collection);
     action->setShortcut( QKeySequence() );
@@ -295,16 +288,16 @@ void MainWindow::setupActions()
     KStandardAction::configureNotifications( this , SLOT(configureNotifications()) , collection  );
     KStandardAction::keyBindings( this , SLOT(showShortcutsDialog()) , collection  );
 
-    action = collection->addAction("configure-profiles");
-    action->setText( i18n("Configure Profiles...") );
+    action = collection->addAction("manage-profiles");
+    action->setText( i18n("Manage Profiles...") );
     action->setIcon( KIcon("configure") );
     connect( action, SIGNAL(triggered()) , this , SLOT(showManageProfilesDialog()) );
 
     // Set up an action and shortcut for activating menu bar.
-    KAction* menuBarAction = collection->addAction("activate_menu");
-    menuBarAction->setText(i18n("Activate Menu"));
-    menuBarAction->setShortcut(QKeySequence(Qt::CTRL+Qt::SHIFT+Qt::Key_F10));
-    connect(menuBarAction, SIGNAL(triggered()), this, SLOT(activateMenuBar()));
+    action = collection->addAction("activate-menu");
+    action->setText(i18n("Activate Menu"));
+    action->setShortcut(QKeySequence(Qt::CTRL+Qt::SHIFT+Qt::Key_F10));
+    connect(action, SIGNAL(triggered()), this, SLOT(activateMenuBar()));
 }
 
 void MainWindow::viewFullScreen(bool fullScreen)
@@ -327,29 +320,49 @@ void MainWindow::setSessionList(ProfileList* list)
     connect( list , SIGNAL(profileSelected(Profile::Ptr)) , this ,
             SLOT(newFromProfile(Profile::Ptr)) );
 
-    connect( list , SIGNAL(actionsChanged(const QList<QAction*>&)) , this ,
-            SLOT(sessionListChanged(const QList<QAction*>&)) );
+    connect( list , SIGNAL(actionsChanged(QList<QAction*>)) , this ,
+            SLOT(sessionListChanged(QList<QAction*>)) );
 }
 
 void MainWindow::sessionListChanged(const QList<QAction*>& actions)
 {
-    // Update the 'New Tab' KActionMenu
-    KMenu *newTabMenu = _newTabMenuAction->menu();
-    newTabMenu->clear();
-    foreach (QAction *action, actions) {
-        newTabMenu->addAction(action);
+    // If only 1 profile is to be shown in the menu, only display
+    // it if it is the non-default profile.
+    if (actions.size() > 2)
+    {
+        // Update the 'New Tab' KActionMenu
+        KMenu* newTabMenu = _newTabMenuAction->menu();
+        newTabMenu->clear();
+        foreach (QAction* action, actions) {
+            newTabMenu->addAction(action);
 
-        // NOTE: _defaultProfile seems to not work here, sigh.
-        Profile::Ptr profile = SessionManager::instance()->defaultProfile();
-        if (profile && profile->name() == action->text()) {
-            action->setIcon(KIcon(profile->icon(), NULL, QStringList("emblem-favorite")));
-            newTabMenu->setDefaultAction(action);
-            QFont font = action->font();
-            font.setBold(true);
-            action->setFont(font);
+            // NOTE: _defaultProfile seems to not work here, sigh.
+            Profile::Ptr profile = SessionManager::instance()->defaultProfile();
+            if (profile && profile->name() == action->text().remove('&')) {
+                action->setIcon(KIcon(profile->icon(), NULL, QStringList("emblem-favorite")));
+                newTabMenu->setDefaultAction(action);
+                QFont font = action->font();
+                font.setBold(true);
+                action->setFont(font);
+            }
         }
     }
+    else
+    {
+        KMenu* newTabMenu = _newTabMenuAction->menu();
+        newTabMenu->clear();
+        Profile::Ptr profile = SessionManager::instance()->defaultProfile();
 
+        // NOTE: Compare names w/o any '&'
+        if (actions.size() == 2 &&  actions[1]->text().remove('&') != profile->name())
+        {
+            newTabMenu->addAction(actions[1]);
+        }
+        else
+        {
+            delete newTabMenu;
+        }
+    }
 }
 
 QString MainWindow::activeSessionDir() const
@@ -415,7 +428,7 @@ bool MainWindow::queryClose()
         if (_pluggedController && _pluggedController->session())
         {
             disconnectController(_pluggedController);
-            _pluggedController->session()->close();
+            _pluggedController->closeSession();
         }
         return false;
     case KMessageBox::Cancel:
@@ -434,7 +447,7 @@ void MainWindow::saveProperties(KConfigGroup& group)
 
 void MainWindow::readProperties(const KConfigGroup& group)
 {
-    SessionManager *manager = SessionManager::instance();
+    SessionManager* manager = SessionManager::instance();
     QString profilePath = group.readPathEntry("Default Profile", QString());
     Profile::Ptr profile = manager->defaultProfile();
     if (!profilePath.isEmpty()) 
@@ -543,17 +556,37 @@ void MainWindow::configureNotifications()
     KNotifyConfigWidget::configure( this );
 }
 
-void MainWindow::showEvent(QShowEvent *event)
+void MainWindow::showEvent(QShowEvent* event)
 {
-    // This code from Konqueror.
-    // We need to check if our toolbars are shown/hidden here, and set
-    // our menu items accordingly. We can't do it in the constructor because
-    // view profiles store toolbar info, and that info is read after
-    // construct time.
-    _toggleMenuBarAction->setChecked( !menuBar()->isHidden() );
+    // Make sure the 'initial' visibility is applied only once.
+    if ( ! _menuBarInitialVisibilityApplied )
+    {
+        // TODO: this is a workaround, not a simple and clear solution.
+        //
+        // Since the geometry restoration of MainWindow happens after
+        // setMenuBarInitialVisibility() is triggered, the initial visibility of
+        // menubar should be applied at this last moment. Otherwise, the initial
+        // visibility will be determined by what is stored in konsolerc, but not
+        // by the selected profile.
+        //
+        menuBar()->setVisible( _menuBarInitialVisibility );
+        _toggleMenuBarAction->setChecked( _menuBarInitialVisibility );
+
+        _menuBarInitialVisibilityApplied = true;
+    }
+
     // Call parent method
     KXmlGuiWindow::showEvent(event);
 }
 
+bool MainWindow::focusNextPrevChild( bool )
+{
+    // In stand-alone konsole, always disable implicit focus switching
+    // through 'Tab' and 'Shift+Tab'
+    //
+    // Kpart is another different story
+    return false;
+}
+
 #include "MainWindow.moc"
 
