@@ -1,6 +1,6 @@
 /*
     This file is part of Konsole, an X terminal.
-    
+
     Copyright 2007-2008 by Robert Knight <robert.knight@gmail.com>
     Copyright 1997,1998 by Lars Doelle <lars.doelle@on-line.de>
 
@@ -37,10 +37,9 @@
     void scrolllock_set_on();
 #endif
 
-// Standard 
+// Standard
 #include <stdio.h>
 #include <unistd.h>
-#include <assert.h>
 
 // Qt
 #include <QtCore/QEvent>
@@ -48,8 +47,8 @@
 #include <QtCore/QByteRef>
 
 // KDE
-#include <kdebug.h>
-#include <klocale.h>
+#include <KDebug>
+#include <KLocale>
 
 // Konsole
 #include "KeyboardTranslator.h"
@@ -59,6 +58,28 @@
 
 using namespace Konsole;
 
+/*
+   The VT100 has 32 special graphical characters. The usual vt100 extended
+   xterm fonts have these at 0x00..0x1f.
+
+   QT's iso mapping leaves 0x00..0x7f without any changes. But the graphicals
+   come in here as proper unicode characters.
+
+   We treat non-iso10646 fonts as VT100 extended and do the requiered mapping
+   from unicode to 0x00..0x1f. The remaining translation is then left to the
+   QCodec.
+*/
+
+// assert for i in [0..31] : vt100extended(vt100_graphics[i]) == i.
+
+unsigned short Konsole::vt100_graphics[32] =
+{ // 0/8     1/9    2/10    3/11    4/12    5/13    6/14    7/15
+  0x0020, 0x25C6, 0x2592, 0x2409, 0x240c, 0x240d, 0x240a, 0x00b0,
+  0x00b1, 0x2424, 0x240b, 0x2518, 0x2510, 0x250c, 0x2514, 0x253c,
+  0xF800, 0xF801, 0x2500, 0xF803, 0xF804, 0x251c, 0x2524, 0x2534,
+  0x252c, 0x2502, 0x2264, 0x2265, 0x03C0, 0x2260, 0x00A3, 0x00b7
+};
+
 Vt102Emulation::Vt102Emulation() 
     : Emulation(),
      _titleUpdateTimer(new QTimer(this))
@@ -96,7 +117,7 @@ void Vt102Emulation::reset()
     setCodec(currentCodec);
   else
     setCodec(LocaleCodec);
- 
+
   bufferedUpdate();
 }
 
@@ -174,7 +195,7 @@ void Vt102Emulation::reset()
 #define TY_CSI_PG(A)  TY_CONSTRUCT(9,A,0)
 #define TY_CSI_PE(A)  TY_CONSTRUCT(10,A,0)
 
-#define MAX_ARGUMENT 4096
+const int MAX_ARGUMENT = 4096 ;
 
 // Tokenizer --------------------------------------------------------------- --
 
@@ -258,7 +279,7 @@ void Vt102Emulation::initTokenizer()
    - P is the length of the token scanned so far.
    - L (often P-1) is the position on which contents we base a decision.
    - C is a character or a group of characters (taken from 'charClass').
-   
+
    - 'cc' is the current character
    - 's' is a pointer to the start of the token buffer
    - 'p' is the current position within the token buffer 
@@ -403,12 +424,12 @@ void Vt102Emulation::processWindowAttributeChange()
     reportDecodingError(); 
     return; 
   }
-  
+
   QString newValue;
   newValue.reserve(tokenBufferPos-i-2);
   for (int j = 0; j < tokenBufferPos-i-2; j++)
     newValue[j] = tokenBuffer[i+1+j];
- 
+
   _pendingTitleUpdates[attributeToChange] = newValue;
   _titleUpdateTimer->start(20);
 }
@@ -720,14 +741,14 @@ void Vt102Emulation::processToken(int token, int p, int q)
     // SET_BTN_EVENT_MOUSE         1002
     // SET_ANY_EVENT_MOUSE         1003
     //
-    
+
     //Note about mouse modes:
     //There are four mouse modes which xterm-compatible terminals can support - 1000,1001,1002,1003
     //Konsole currently supports mode 1000 (basic mouse press and release) and mode 1002 (dragging the mouse).
     //TODO:  Implementation of mouse modes 1001 (something called hilight tracking) and 
     //1003 (a slight variation on dragging the mouse)
     //
- 
+
     case TY_CSI_PR('h', 1000) :          setMode      (MODE_Mouse1000); break; //XTERM
     case TY_CSI_PR('l', 1000) :        resetMode      (MODE_Mouse1000); break; //XTERM
     case TY_CSI_PR('s', 1000) :         saveMode      (MODE_Mouse1000); break; //XTERM
@@ -952,16 +973,23 @@ void Vt102Emulation::sendKeyEvent( QKeyEvent* event )
 
         if ( entry.command() != KeyboardTranslator::NoCommand )
         {
+            TerminalDisplay * currentView = _currentScreen->currentTerminalDisplay();
+
             if (entry.command() & KeyboardTranslator::EraseCommand) {
                 textToSend += eraseChar();
             } else if ( entry.command() & KeyboardTranslator::ScrollPageUpCommand )
-                _currentScreen->currentTerminalDisplay()->scrollScreenWindow( ScreenWindow::ScrollPages , -1 );
+                currentView->scrollScreenWindow( ScreenWindow::ScrollPages , -1 );
             else if ( entry.command() & KeyboardTranslator::ScrollPageDownCommand )
-                _currentScreen->currentTerminalDisplay()->scrollScreenWindow( ScreenWindow::ScrollPages , 1 );
+                currentView->scrollScreenWindow( ScreenWindow::ScrollPages , 1 );
             else if ( entry.command() & KeyboardTranslator::ScrollLineUpCommand )
-                _currentScreen->currentTerminalDisplay()->scrollScreenWindow( ScreenWindow::ScrollLines , -1 );
+                currentView->scrollScreenWindow( ScreenWindow::ScrollLines , -1 );
             else if ( entry.command() & KeyboardTranslator::ScrollLineDownCommand )
-                _currentScreen->currentTerminalDisplay()->scrollScreenWindow( ScreenWindow::ScrollLines , 1 );
+                currentView->scrollScreenWindow( ScreenWindow::ScrollLines , 1 );
+            else if ( entry.command() & KeyboardTranslator::ScrollUpToTopCommand )
+                currentView->scrollScreenWindow( ScreenWindow::ScrollLines ,
+                                                - currentView->screenWindow()->currentLine());
+            else if ( entry.command() & KeyboardTranslator::ScrollDownToBottomCommand)
+                currentView->scrollScreenWindow( ScreenWindow::ScrollLines , lineCount());
         }
         else if ( !entry.text().isEmpty() ) 
         {
